@@ -1,10 +1,10 @@
 import click
 import numpy as np
 import re
+import tifffile
+import xtiff
 
 from pathlib import Path
-from tifffile import imwrite
-from xtiff import to_tiff
 
 from steinbock import io
 from steinbock.export._cli.data import anndata_cmd, csv_cmd, fcs_cmd
@@ -56,12 +56,16 @@ def ome_cmd(img_dir, panel_file, ome_dir):
     ]
     Path(ome_dir).mkdir(exist_ok=True)
     for img_file in io.list_image_files(img_dir):
-        img = io.read_image(img_file)
+        img = io.read_image(img_file, native_dtype=True)
         ome_file = io._as_path_with_suffix(
             Path(ome_dir) / img_file.name, ".ome.tiff"
         )
-        to_tiff(img, ome_file, channel_names=channel_names)
-        click.echo(ome_file.name)
+        xtiff.to_tiff(
+            io._to_dtype(img, np.float32),
+            ome_file,
+            channel_names=channel_names,
+        )
+        click.echo(ome_file)
         del img
 
 
@@ -120,20 +124,29 @@ def histocat_cmd(img_dir, mask_dir, panel_file, histocat_dir):
         for channel_name, channel_img in zip(channel_names, img):
             channel_name = re.sub(r"\s*/\s*", "_", channel_name)
             channel_name = re.sub(r"\s", "_", channel_name)
-            imwrite(
-                histocat_img_dir / f"{channel_name}.tiff",
-                data=io._to_dtype(channel_img, np.float32),
+            histocat_img_file = histocat_img_dir / f"{channel_name}.tiff"
+            tifffile.imwrite(
+                histocat_img_file,
+                data=io._to_dtype(channel_img, np.float32)[
+                    np.newaxis, np.newaxis, np.newaxis, :, :, np.newaxis
+                ],
                 imagej=True,
             )
+            click.echo(histocat_img_file)
         mask = None
         if mask_files is not None:
-            mask = io.read_mask(mask_files[i])
-            imwrite(
-                histocat_img_dir / f"{img_file.stem}_mask.tiff",
-                data=io._to_dtype(mask, np.uint16),
+            mask = io.read_mask(mask_files[i], native_dtype=True)
+            histocat_mask_file = (
+                histocat_img_dir / f"{img_file.stem}_mask.tiff"
+            )
+            tifffile.imwrite(
+                histocat_mask_file,
+                data=io._to_dtype(mask, np.uint16)[
+                    np.newaxis, np.newaxis, np.newaxis, :, :, np.newaxis
+                ],
                 imagej=True,
             )
-        click.echo(img_file.name)
+            click.echo(histocat_mask_file)
         del img, mask
 
 
