@@ -23,6 +23,13 @@ from steinbock.measurement.regionprops import try_measure_regionprops_from_disk
     show_default=True,
     help="Path to the mask directory",
 )
+@click.option(
+    "--mmap/--no-mmap",
+    "mmap",
+    default=False,
+    show_default=True,
+    help="Use memory mapping for reading images/masks",
+)
 @click.argument("skimage_regionprops", nargs=-1, type=click.STRING)
 @click.option(
     "-o",
@@ -32,7 +39,9 @@ from steinbock.measurement.regionprops import try_measure_regionprops_from_disk
     show_default=True,
     help="Path to the object region properties output directory",
 )
-def regionprops_cmd(img_dir, mask_dir, skimage_regionprops, regionprops_dir):
+def regionprops_cmd(
+    img_dir, mask_dir, mmap, skimage_regionprops, regionprops_dir
+):
     img_files = io.list_image_files(img_dir)
     mask_files = io.list_mask_files(mask_dir, base_files=img_files)
     Path(regionprops_dir).mkdir(exist_ok=True)
@@ -45,9 +54,11 @@ def regionprops_cmd(img_dir, mask_dir, skimage_regionprops, regionprops_dir):
             "eccentricity",
         ]
     for img_file, mask_file, regionprops in try_measure_regionprops_from_disk(
-        img_files, mask_files, skimage_regionprops
+        img_files, mask_files, skimage_regionprops, mmap=mmap
     ):
-        regionprops_stem = Path(regionprops_dir) / img_file.stem
-        regionprops_file = io.write_data(regionprops, regionprops_stem)
+        regionprops_file = io._as_path_with_suffix(
+            Path(regionprops_dir) / img_file.name, ".csv"
+        )
+        io.write_data(regionprops, regionprops_file)
         click.echo(regionprops_file)
         del regionprops
