@@ -15,9 +15,11 @@ class TestDataExport:
             imc_test_data_steinbock_path / "regionprops",
             base_files=intensities_files,
         )
-        data.try_convert_to_dataframe_from_disk(
+        gen = data.try_convert_to_dataframe_from_disk(
             intensities_files, regionprops_files
-        )  # TODO
+        )
+        for img_file_name, data_files, df in gen:
+            pass  # TODO
 
     def test_try_convert_to_anndata_from_disk(
         self, imc_test_data_steinbock_path: Path
@@ -29,8 +31,20 @@ class TestDataExport:
             imc_test_data_steinbock_path / "regionprops",
             base_files=intensities_files,
         )
-        gen = data.try_convert_to_anndata_from_disk(
-            intensities_files, regionprops_files
+        neighbors_files = io.list_neighbors_files(
+            imc_test_data_steinbock_path / "neighbors",
+            base_files=intensities_files,
         )
-        for intensities_file, (regionprops_file,), anndata in gen:
+        gen = data.try_convert_to_anndata_from_disk(
+            intensities_files,
+            regionprops_files,
+            neighbors_files=neighbors_files,
+        )
+        for (
+            img_file_name,
+            intensities_file,
+            (regionprops_file, ),
+            neighbors_file,
+            adata,
+        ) in gen:
             pass  # TODO
