@@ -4,10 +4,11 @@ import pytest
 from pathlib import Path
 
 from steinbock import io
-from steinbock._env import keras_models_dir
 from steinbock.segmentation import deepcell
 from steinbock.segmentation.deepcell import Application
 
+keras_models_dir = "/opt/keras/models"
+
 
 @pytest.mark.skipif(
     not deepcell.deepcell_available, reason="DeepCell is not available"
